from typing import Dict

from sparksampling.sample.base_sampling import SparkBaseSamplingJob
import random
import json


class StratifiedSamplingImp(SparkBaseSamplingJob):
    cls_args = ['fraction', 'with_replacement', 'stratified_key', 'seed', 'count', 'sampling_col']

    def __init__(self, *args, **kwargs):
        super(StratifiedSamplingImp, self).__init__(*args, **kwargs)
        self.fraction = self._init_fraction(kwargs.pop('fraction'))
        self.stratified_key = kwargs.pop('stratified_key')
        self.with_replacement = kwargs.pop('with_replacement', False)
        self.seed = kwargs.pop('seed', random.randint(1, 65535))
        self.count = kwargs.pop('count', 0)
        self.sampling_col = list(kwargs.pop('sampling_col', []))

    @staticmethod
    def _init_fraction(fraction_str):
        fraction = json.loads(fraction_str)
        if isinstance(fraction, int):
            fraction = float(fraction)
        if not isinstance(fraction, (Dict, float)):
            raise ValueError("fraction must be float or dict")
        return fraction

    def run(self, df):
        fraction = self._convert_fraction_to_dict(df, self.stratified_key, self.fraction)
        if self.sampling_col:
            df = df[self.sampling_col]
        df = df.sampleBy(col=self.stratified_key, fractions=fraction, seed=self.seed)
        if self.count:
            df = df.limit(self.count)
        return df

    @staticmethod
    def _convert_fraction_to_dict(df, stratified_key, fraction) -> Dict:
        if isinstance(fraction, Dict):
            return fraction

        y = df.select(stratified_key)
        labels = y.distinct().toPandas().to_numpy().reshape(-1)
        convert_fraction = dict()
        for label in labels:
            convert_fraction[label] = fraction
        StratifiedSamplingImp.log.info(f"Convert fraction {fraction} to dict", convert_fraction)
        return convert_fraction



from pprint import pformat
from typing import List, Dict

from pyspark.sql import DataFrame

from sparksampling.engine.base_engine import SparkBaseEngine, record_job_id
from sparksampling.engine_factory import EngineFactory
from sparksampling.error import BadParamError
from sparksampling.file_format import FileFormatFactory
from sparksampling.mixin import acquire_worker, LogMixin
from sparksampling.proto.sampling_service_pb2 import RelationSamplingRequest
from sparksampling.sample import SamplingFactory
from sparksampling.utilities import check_spark_session
from graphlib import TopologicalSorter, CycleError


def _check_param(conf):
    def check_duplicated_name(name):
        if name not in name_set:
            name_set.add(name)
        else:
            raise BadParamError(f'Duplicated name: {name}')

    name_set = set()
    for sampling_stage in conf.get('sampling_stages', []):
        check_duplicated_name(sampling_stage['name'])
        if not (sampling_stage.get('input_path') or sampling_stage.get('input_name')):
            raise BadParamError(
                f'Need at least one sampling source(input_path or input_name) for relation {pformat(sampling_stage)}')

    for relation_stage in conf.get('relation_stage', []):
        # check duplicate name
        check_duplicated_name(relation_stage['name'])
        for relation in relation_stage.get('relations', []):
            if not (relation.get('relation_path') or relation.get('relation_name')):
                raise BadParamError(
                    f'Need at least one relation source(relation_path or relation_name) for relation {pformat(relation_stage)}')
            if not (relation.get('relation_col') or relation.get('input_col')):
                raise BadParamError(
                    f'Need at least one col(relation_col or input_col) for relation {pformat(relation_stage)}')


def _set_default(conf):
    # allow empty stage for export or sampling only job
    conf.setdefault('sampling_stages', [])
    conf.setdefault('relation_stages', [])

    default_sampling_method = conf.get('default_sampling_method')
    default_sampling_conf = conf.get('default_sampling_conf')
    default_file_format = conf.get('default_file_format')
    default_format_conf = conf.get('default_format_conf')
    for sampling_stage in conf['sampling_stages']:
        sampling_stage.setdefault('sampling_method', default_sampling_method)
        sampling_stage.setdefault('sampling_conf', default_sampling_conf)
        sampling_stage.setdefault('file_format', default_file_format)
        sampling_stage.setdefault('format_conf', default_format_conf)
    for relation_stage in conf['relation_stages']:
        relation_stage.setdefault('file_format', default_file_format)
        relation_stage.setdefault('format_conf', default_format_conf)
        for relation in relation_stage.get('relations', []):
            if not relation.get('relation_col'):
                relation['relation_col'] = relation['input_col']
            if not relation.get('input_col'):
                relation['input_col'] = relation['relation_col']
            relation.setdefault('file_format', default_file_format)
            relation.setdefault('format_conf', default_format_conf)
    return conf


class JobStage(LogMixin):
    def __init__(self, spark, stage_config, name_df_map):
        self.spark = spark
        self.stage_config = stage_config
        self.name_df_map = name_df_map

        self._df = None
        self.sampled_path = None
        self.file_format_imp = None
        self.sample_imp = None

        self._init_sample_imp()
        self._init_file_format()

    def _init_file_format(self):
        self.file_format_imp = FileFormatFactory.get_file_imp(self.spark, self.file_format, self.format_conf)

    def _init_sample_imp(self):
        if not (self.sampling_method and self.sampling_conf):
            return
        self.sample_imp = SamplingFactory.get_sampling_imp(self.sampling_method, self.sampling_conf)

    def build_dataframe(self):
        self.log.debug(f'Starting building stage {self.name}')
        if self.input_name:
            self._df = self.name_df_map[self.input_name]
        else:
            self._df = self.file_format_imp.read(self.input_path)

        if self.sample_imp:
            self._df = self.sample_imp.run(self._df)

        for relation in self.relations:
            relation_name = relation.get('relation_name')
            if relation_name:
                relation_df = self.name_df_map[relation_name]
            else:
                # using relation_path to construct dataframe
                relation_file_format = FileFormatFactory.get_file_imp(
                    self.spark,
                    relation['file_format'],
                    relation['format_conf']
                )
                # no need to worry if spark read the same csv twice, spark only read once
                relation_df = relation_file_format.read(relation['relation_path'])

            # python's way for self._df.join(relation_df, self._df.input_col == relation_df.relation_col, 'semi')
            cond = [getattr(self._df, relation['input_col']) == getattr(relation_df, relation['relation_col'])]
            self._df = self._df.join(relation_df, cond, how='semi')

        self.name_df_map[self.name] = self._df

    @property
    def relations(self):
        return self.stage_config.get('relations', [])

    @property
    def name(self):
        return self.stage_config.get('name')

    @property
    def sampling_method(self):
        return self.stage_config.get('sampling_method')

    @property
    def sampling_conf(self):
        return self.stage_config.get('sampling_conf')

    @property
    def file_format(self):
        return self.stage_config.get('file_format')

    @property
    def format_conf(self):
        return self.stage_config.get('format_conf')

    @property
    def output_path(self):
        return self.stage_config.get('output_path')

    @property
    def output_col(self):
        return self.stage_config.get('output_col')

    @property
    def input_name(self):
        return self.stage_config.get('input_name')

    @property
    def input_path(self):
        return self.stage_config.get('input_path')

    def export_dataframe(self):
        if self.output_path:
            self.log.debug(f'Exporting stage {self.name} to {self.output_path}')
            self.sampled_path = self.file_format_imp.write(self._df, self.output_path, self.output_col)
        else:
            self.log.debug(f'Nothing to export for {self.name}')


class RMSEngine(SparkBaseEngine):
    # Relationship Mapping Sampling
    def __init__(self,
                 parent,
                 sampling_stages: List[Dict],
                 relation_stages: List[Dict],
                 job_id: str,
                 dry_run: bool,
                 ):
        super(RMSEngine, self).__init__()
        self.parent = parent
        self.sampling_stages = sampling_stages
        self.relation_stages = relation_stages
        self.job_id = job_id
        self.dry_run = dry_run

        self.name_stage_map = dict()
        self._init_name_stage_map()

    def _init_name_stage_map(self):
        for sampling_stage in self.sampling_stages:
            self.name_stage_map[sampling_stage['name']] = sampling_stage
            self.name_stage_map['type'] = 'sample'
        for relation_stages in self.relation_stages:
            self.name_stage_map[relation_stages['name']] = relation_stages
            self.name_stage_map['type'] = 'relation'

    @acquire_worker
    @check_spark_session
    @record_job_id
    def submit(self, *args, **kwargs):
        try:
            build_order = self._get_stage_order()
        except CycleError as e:
            raise BadParamError(f'Loop detected: {str(e)}')
        self.log.info(f'Initializing spark job as {build_order}')

        spark_job_seq = self._process_spark_seq(build_order)
        result = self._get_seq_result(spark_job_seq)
        return result

    def _get_stage_order(self):
        topological_map = dict()

        for sampling_stage in self.sampling_stages:
            rely_set = topological_map.setdefault(sampling_stage['name'], set())
            rely_name = sampling_stage.get('input_name')
            if rely_name:
                rely_set.add(rely_name)

        for relation_stage in self.relation_stages:
            rely_set = topological_map.setdefault(relation_stage['name'], set())
            rely_name = relation_stage.get('input_name')
            if rely_name:
                rely_set.add(rely_name)
            for relation in relation_stage.get('relations', []):
                relation_name = relation.get('relation_name')
                if relation_name:
                    rely_set.add(relation_name)
        self.log.debug(f"topological_map is {pformat(topological_map)}")
        build_other = list(TopologicalSorter(topological_map).static_order())
        return build_other

    def _process_spark_seq(self, build_order):
        job_seq = []
        name_df_map = dict()
        for name in build_order:
            stage = JobStage(self.spark, self.name_stage_map[name], name_df_map)
            stage.build_dataframe()
            job_seq.append(stage)
        if self.dry_run:
            self.log.info('Dry run, skip export dataframe')
            return
        self.log.info('Generated spark job sequence, start exporting dataframe...')
        for stage in job_seq:
            stage.export_dataframe()
        return job_seq

    def _get_seq_result(self, seq):
        self.log.debug('Start collecting result...')
        if self.dry_run:
            self.log.info('Dry run, nothing output')
            return []
        result = []
        for job in seq:
            if job.sampled_path:
                result.append({'name': job.name, 'sampled_path': job.sampled_path})
        self.log.info(f'result collectd: \n{pformat(result)}')
        return result

    @classmethod
    def config(cls, kwargs):
        _check_param(kwargs)
        try:
            kwargs = _set_default(kwargs)
            sampling_stages = kwargs.pop('sampling_stages')
            relation_stages = kwargs.pop('relation_stages')
            job_id = kwargs.pop('job_id')
            dry_run = kwargs.pop('dry_run', False)
        except KeyError as e:
            cls.log.info("Missing required parameters", e)
            raise BadParamError(f"Missing required parameters {str(e)}")

        config_dict = {
            'sampling_stages': sampling_stages,
            'relation_stages': relation_stages,
            'job_id': job_id,
            'dry_run': dry_run,
        }
        cls.log.info(f"Initializing job conf... \n {pformat(config_dict)}")
        return config_dict

    @classmethod
    def is_matching(cls, request_type):
        return request_type == RelationSamplingRequest


EngineFactory.register(RMSEngine)



# import all .py from in this dir
from os.path import dirname, basename, isfile, join
import glob
import importlib

modules = glob.glob(join(dirname(__file__), "*.py"))
sub_packages = (basename(f)[:-3] for f in modules if isfile(f) and not f.endswith('__init__.py'))
packages = (str(__package__) + '.' + i for i in sub_packages)
[importlib.import_module(p) for p in packages]



from sparksampling.error import BadParamError

from sparksampling.proto.sampling_service_pb2 import (
    FILE_FORMAT_CSV
)
from sparksampling.file_format.base_file_format import SparkBaseFileFormat
from sparksampling.file_format.csv_file_imp import CsvFileImpSpark

from sparksampling.mixin import LogMixin


class FileFormatFactory(LogMixin):
    format_map = {
        FILE_FORMAT_CSV: CsvFileImpSpark,
    }

    @classmethod
    def _get_imp_class(cls, file_format):
        imp_class = cls.format_map.get(file_format, None)
        if not imp_class:
            raise BadParamError(f"No matching file format: {file_format}")
        return imp_class

    @classmethod
    def _get_format_conf(cls, file_format, format_conf):
        return cls.format_map[file_format].get_init_conf(format_conf)

    @classmethod
    def get_file_imp(cls, spark, file_format, format_conf) -> SparkBaseFileFormat:
        return cls._get_imp_class(file_format)(spark, **cls._get_format_conf(file_format, format_conf))



import os
import uuid
from concurrent.futures import ThreadPoolExecutor as PoolExecutor

from sparksampling.error import ProcessError
import pandas as pd

from sparksampling.mixin import LogMixin


class LocalAdapterMixin(LogMixin):
    MERGE_CHUNK_SIZE = 0
    executor = PoolExecutor(os.cpu_count() or 1)

    @staticmethod
    def _merge_local_csv(dir_prefix, csv_files: list, sep):
        merge_file_name = csv_files.pop(0)
        merged_csv_path = os.path.join(dir_prefix, merge_file_name)

        def append_to_file(chunk_file):
            chunk_file_path = os.path.join(dir_prefix, chunk_file)
            with pd.read_csv(chunk_file_path, sep=sep, chunksize=100000, dtype='string') as reader:
                for chunk in reader:
                    chunk.to_csv(merged_csv_path, sep=sep, mode='a', index=False, )
            os.remove(chunk_file_path)

        list(map(append_to_file, csv_files))
        return merge_file_name

    @classmethod
    def _merge_local_csv_parallel(cls, abs_output_path, merge_file_names, sep):
        if len(merge_file_names) == 1:
            return os.path.join(abs_output_path, merge_file_names[0])
        pool = []
        for i in range(0, len(merge_file_names), cls.MERGE_CHUNK_SIZE):
            p = cls.executor.submit(cls._merge_local_csv,
                                    abs_output_path=abs_output_path,
                                    csv_files=merge_file_names[i:i + cls.MERGE_CHUNK_SIZE],
                                    sep=sep)
            pool.append(p)

        merge_file_names = [p.result() for p in pool]

        return cls._merge_local_csv_parallel(abs_output_path, merge_file_names, sep)

    @classmethod
    def _merge_local_file(cls, abs_output_path, csv_files, sep):

        if not cls.MERGE_CHUNK_SIZE or cls.MERGE_CHUNK_SIZE == 1:
            return os.path.join(abs_output_path,
                                cls._merge_local_csv(dir_prefix=abs_output_path, csv_files=csv_files, sep=sep))
        else:
            return cls._merge_local_csv_parallel(abs_output_path, csv_files, sep)

    @classmethod
    def _get_local_sampled_file(cls, output_path, sep):
        abs_dir = os.path.abspath(output_path)
        csv_files = sorted([filename for filename in os.listdir(abs_dir) if filename.endswith('.csv')])
        if not csv_files:
            raise ProcessError('csv file not found')
        if len(csv_files) > 1:
            if os.getenv('NO_REPARTITION'):
                cls.log.info(f"No repartition and no merge for files:  {csv_files[:10]}...(only show top 10)")
                return abs_dir
            cls.log.info(f"Start merging sampling files {len(csv_files)}: {csv_files[:10]}...(only show top 10)", )
            csv_file = cls._merge_local_file(abs_dir, csv_files, sep)
        else:
            csv_file = csv_files[0]
        cls.log.debug(f"Output one file: {csv_file}")
        return os.path.join(abs_dir, csv_file)


class S3OutputAdapterMixin(LogMixin):
    MERGE_CHUNK_SIZE = 0
    executor = PoolExecutor(os.cpu_count() or 1)
    s3_theme = 's3a://'

    @classmethod
    def _get_s3_sampled_file(cls, output_path, sep, spark):
        def get_s3_conf():
            spark_conf = spark.sparkContext.getConf()
            return {
                'aws_access_key_id': spark_conf.get('spark.hadoop.fs.s3a.access.key'),
                'aws_secret_access_key': spark_conf.get('spark.hadoop.fs.s3a.secret.key'),
                'endpoint_url': spark_conf.get('spark.hadoop.fs.s3a.endpoint')
            }

        # Experimental function, no need for now
        import boto3
        session = boto3.resource('s3', **get_s3_conf(), verify=False)
        s3_path = output_path.replace(cls.s3_theme, '')
        bucket_name, dir_path = s3_path.split('/', 1)
        bucket = session.Bucket(bucket_name)

        csv_files = sorted([
            object_summary.key for object_summary in bucket.objects.filter(Prefix=dir_path)
            if object_summary.key.endswith('.csv')
        ])

        if len(csv_files) > 1:
            cls.log.info(f"Generate multiple csv files, s3 does not support automatic merging yet: {csv_files}", )
            return output_path
        else:
            csv_file = csv_files[0]
        cls.log.debug(f"Output one file: {csv_file}")
        return cls.s3_theme + '/'.join([bucket_name, csv_file])


class OutputAdapterMixin(LocalAdapterMixin, S3OutputAdapterMixin):
    MERGE_CHUNK_SIZE = 0
    executor = PoolExecutor(os.cpu_count() or 1)

    @classmethod
    def _get_sampled_file(cls, output_path, sep, spark=None):
        if output_path.startswith(cls.s3_theme):
            return cls._get_s3_sampled_file(output_path, sep, spark)
        if os.path.exists(output_path):
            return cls._get_local_sampled_file(output_path, sep)
        cls.log.info(f"Unsupported file path conversion method, returns the original path: {output_path}")
        return output_path



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sampling_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16sampling_service.proto\"\xea\x01\n\x0fSamplingRequest\x12,\n\x0fsampling_method\x18\x01 \x01(\x0e\x32\x13.SamplingMethodType\x12$\n\x0b\x66ile_format\x18\x02 \x01(\x0e\x32\x0f.FileFormatType\x12$\n\rsampling_conf\x18\x03 \x01(\x0b\x32\r.SamplingConf\x12$\n\x0b\x66ormat_conf\x18\x04 \x01(\x0b\x32\x0f.FileFormatConf\x12\x12\n\ninput_path\x18\x05 \x01(\t\x12\x13\n\x0boutput_path\x18\x06 \x01(\t\x12\x0e\n\x06job_id\x18\x07 \x01(\t\"\x1f\n\rCancelRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\"/\n\x0e\x43\x61ncelResponse\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\"\xaf\x01\n\x10SamplingResponse\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12,\n\x04\x64\x61ta\x18\x03 \x01(\x0b\x32\x1e.SamplingResponse.ResponseData\x1aN\n\x0cResponseData\x12(\n\x0eparent_request\x18\x01 \x01(\x0b\x32\x10.SamplingRequest\x12\x14\n\x0csampled_path\x18\x02 \x01(\t\"#\n\x04\x64ict\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\x85\x01\n\x0cSamplingConf\x12\x10\n\x08\x66raction\x18\x01 \x01(\t\x12\x18\n\x10with_replacement\x18\x02 \x01(\x08\x12\x0c\n\x04seed\x18\x03 \x01(\r\x12\x16\n\x0estratified_key\x18\x04 \x01(\t\x12\r\n\x05\x63ount\x18\x05 \x01(\r\x12\x14\n\x0csampling_col\x18\x06 \x03(\t\"2\n\x0e\x46ileFormatConf\x12\x13\n\x0bwith_header\x18\x01 \x01(\x08\x12\x0b\n\x03sep\x18\x02 \x01(\t\"\xb0\x08\n\x17RelationSamplingRequest\x12?\n\x0fsampling_stages\x18\x01 \x03(\x0b\x32&.RelationSamplingRequest.SamplingStage\x12?\n\x0frelation_stages\x18\x02 \x03(\x0b\x32&.RelationSamplingRequest.RelationStage\x12\x0e\n\x06job_id\x18\x03 \x01(\t\x12\x34\n\x17\x64\x65\x66\x61ult_sampling_method\x18\x04 \x01(\x0e\x32\x13.SamplingMethodType\x12,\n\x15\x64\x65\x66\x61ult_sampling_conf\x18\x05 \x01(\x0b\x32\r.SamplingConf\x12,\n\x13\x64\x65\x66\x61ult_file_format\x18\x06 \x01(\x0e\x32\x0f.FileFormatType\x12,\n\x13\x64\x65\x66\x61ult_format_conf\x18\x07 \x01(\x0b\x32\x0f.FileFormatConf\x12\x0f\n\x07\x64ry_run\x18\x08 \x01(\x08\x1a\x8e\x02\n\rSamplingStage\x12\x12\n\ninput_path\x18\x01 \x01(\t\x12\x12\n\ninput_name\x18\x02 \x01(\t\x12\x13\n\x0boutput_path\x18\x03 \x01(\t\x12\x12\n\noutput_col\x18\x04 \x03(\t\x12\x0c\n\x04name\x18\x05 \x01(\t\x12,\n\x0fsampling_method\x18\x06 \x01(\x0e\x32\x13.SamplingMethodType\x12$\n\rsampling_conf\x18\x07 \x01(\x0b\x32\r.SamplingConf\x12$\n\x0b\x66ile_format\x18\x08 \x01(\x0e\x32\x0f.FileFormatType\x12$\n\x0b\x66ormat_conf\x18\t \x01(\x0b\x32\x0f.FileFormatConf\x1a\xad\x01\n\x08Relation\x12\x15\n\rrelation_path\x18\x01 \x01(\t\x12\x15\n\rrelation_name\x18\x02 \x01(\t\x12\x14\n\x0crelation_col\x18\x03 \x01(\t\x12\x11\n\tinput_col\x18\x04 \x01(\t\x12$\n\x0b\x66ile_format\x18\x05 \x01(\x0e\x32\x0f.FileFormatType\x12$\n\x0b\x66ormat_conf\x18\x06 \x01(\x0b\x32\x0f.FileFormatConf\x1a\xf0\x01\n\rRelationStage\x12\x12\n\ninput_path\x18\x01 \x01(\t\x12\x12\n\ninput_name\x18\x02 \x01(\t\x12\x13\n\x0boutput_path\x18\x03 \x01(\t\x12\x12\n\noutput_col\x18\x04 \x03(\t\x12\x0c\n\x04name\x18\x05 \x01(\t\x12\x34\n\trelations\x18\x06 \x03(\x0b\x32!.RelationSamplingRequest.Relation\x12$\n\x0b\x66ile_format\x18\x07 \x01(\x0e\x32\x0f.FileFormatType\x12$\n\x0b\x66ormat_conf\x18\x08 \x01(\x0b\x32\x0f.FileFormatConf\"\x9d\x02\n\x18RelationSamplingResponse\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x34\n\x04\x64\x61ta\x18\x03 \x01(\x0b\x32&.RelationSamplingResponse.ResponseData\x1a\x32\n\x0cSampleResult\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0csampled_path\x18\x02 \x01(\t\x1ax\n\x0cResponseData\x12\x30\n\x0eparent_request\x18\x01 \x01(\x0b\x32\x18.RelationSamplingRequest\x12\x36\n\x06result\x18\x02 \x03(\x0b\x32&.RelationSamplingResponse.SampleResult*\x80\x01\n\x12SamplingMethodType\x12\x12\n\x0eUNKNOWN_METHOD\x10\x00\x12\x1a\n\x16RANDOM_SAMPLING_METHOD\x10\x01\x12\x1e\n\x1aSTRATIFIED_SAMPLING_METHOD\x10\x02\x12\x1a\n\x16SIMPLE_SAMPLING_METHOD\x10\x03*9\n\x0e\x46ileFormatType\x12\x12\n\x0eUNKNOWN_FORMAT\x10\x00\x12\x13\n\x0f\x46ILE_FORMAT_CSV\x10\x01\x32\xca\x01\n\x14SparkSamplingService\x12\x34\n\x0bSamplingJob\x12\x10.SamplingRequest\x1a\x11.SamplingResponse\"\x00\x12L\n\x13RelationSamplingJob\x12\x18.RelationSamplingRequest\x1a\x19.RelationSamplingResponse\"\x00\x12.\n\tCancelJob\x12\x0e.CancelRequest\x1a\x0f.CancelResponse\"\x00\x62\x06proto3')

_SAMPLINGMETHODTYPE = DESCRIPTOR.enum_types_by_name['SamplingMethodType']
SamplingMethodType = enum_type_wrapper.EnumTypeWrapper(_SAMPLINGMETHODTYPE)
_FILEFORMATTYPE = DESCRIPTOR.enum_types_by_name['FileFormatType']
FileFormatType = enum_type_wrapper.EnumTypeWrapper(_FILEFORMATTYPE)
UNKNOWN_METHOD = 0
RANDOM_SAMPLING_METHOD = 1
STRATIFIED_SAMPLING_METHOD = 2
SIMPLE_SAMPLING_METHOD = 3
UNKNOWN_FORMAT = 0
FILE_FORMAT_CSV = 1


_SAMPLINGREQUEST = DESCRIPTOR.message_types_by_name['SamplingRequest']
_CANCELREQUEST = DESCRIPTOR.message_types_by_name['CancelRequest']
_CANCELRESPONSE = DESCRIPTOR.message_types_by_name['CancelResponse']
_SAMPLINGRESPONSE = DESCRIPTOR.message_types_by_name['SamplingResponse']
_SAMPLINGRESPONSE_RESPONSEDATA = _SAMPLINGRESPONSE.nested_types_by_name['ResponseData']
_DICT = DESCRIPTOR.message_types_by_name['dict']
_SAMPLINGCONF = DESCRIPTOR.message_types_by_name['SamplingConf']
_FILEFORMATCONF = DESCRIPTOR.message_types_by_name['FileFormatConf']
_RELATIONSAMPLINGREQUEST = DESCRIPTOR.message_types_by_name['RelationSamplingRequest']
_RELATIONSAMPLINGREQUEST_SAMPLINGSTAGE = _RELATIONSAMPLINGREQUEST.nested_types_by_name['SamplingStage']
_RELATIONSAMPLINGREQUEST_RELATION = _RELATIONSAMPLINGREQUEST.nested_types_by_name['Relation']
_RELATIONSAMPLINGREQUEST_RELATIONSTAGE = _RELATIONSAMPLINGREQUEST.nested_types_by_name['RelationStage']
_RELATIONSAMPLINGRESPONSE = DESCRIPTOR.message_types_by_name['RelationSamplingResponse']
_RELATIONSAMPLINGRESPONSE_SAMPLERESULT = _RELATIONSAMPLINGRESPONSE.nested_types_by_name['SampleResult']
_RELATIONSAMPLINGRESPONSE_RESPONSEDATA = _RELATIONSAMPLINGRESPONSE.nested_types_by_name['ResponseData']
SamplingRequest = _reflection.GeneratedProtocolMessageType('SamplingRequest', (_message.Message,), {
  'DESCRIPTOR' : _SAMPLINGREQUEST,
  '__module__' : 'sampling_service_pb2'
  # @@protoc_insertion_point(class_scope:SamplingRequest)
  })
_sym_db.RegisterMessage(SamplingRequest)

CancelRequest = _reflection.GeneratedProtocolMessageType('CancelRequest', (_message.Message,), {
  'DESCRIPTOR' : _CANCELREQUEST,
  '__module__' : 'sampling_service_pb2'
  # @@protoc_insertion_point(class_scope:CancelRequest)
  })
_sym_db.RegisterMessage(CancelRequest)

CancelResponse = _reflection.GeneratedProtocolMessageType('CancelResponse', (_message.Message,), {
  'DESCRIPTOR' : _CANCELRESPONSE,
  '__module__' : 'sampling_service_pb2'
  # @@protoc_insertion_point(class_scope:CancelResponse)
  })
_sym_db.RegisterMessage(CancelResponse)

SamplingResponse = _reflection.GeneratedProtocolMessageType('SamplingResponse', (_message.Message,), {

  'ResponseData' : _reflection.GeneratedProtocolMessageType('ResponseData', (_message.Message,), {
    'DESCRIPTOR' : _SAMPLINGRESPONSE_RESPONSEDATA,
    '__module__' : 'sampling_service_pb2'
    # @@protoc_insertion_point(class_scope:SamplingResponse.ResponseData)
    })
  ,
  'DESCRIPTOR' : _SAMPLINGRESPONSE,
  '__module__' : 'sampling_service_pb2'
  # @@protoc_insertion_point(class_scope:SamplingResponse)
  })
_sym_db.RegisterMessage(SamplingResponse)
_sym_db.RegisterMessage(SamplingResponse.ResponseData)

dict = _reflection.GeneratedProtocolMessageType('dict', (_message.Message,), {
  'DESCRIPTOR' : _DICT,
  '__module__' : 'sampling_service_pb2'
  # @@protoc_insertion_point(class_scope:dict)
  })
_sym_db.RegisterMessage(dict)

SamplingConf = _reflection.GeneratedProtocolMessageType('SamplingConf', (_message.Message,), {
  'DESCRIPTOR' : _SAMPLINGCONF,
  '__module__' : 'sampling_service_pb2'
  # @@protoc_insertion_point(class_scope:SamplingConf)
  })
_sym_db.RegisterMessage(SamplingConf)

FileFormatConf = _reflection.GeneratedProtocolMessageType('FileFormatConf', (_message.Message,), {
  'DESCRIPTOR' : _FILEFORMATCONF,
  '__module__' : 'sampling_service_pb2'
  # @@protoc_insertion_point(class_scope:FileFormatConf)
  })
_sym_db.RegisterMessage(FileFormatConf)

RelationSamplingRequest = _reflection.GeneratedProtocolMessageType('RelationSamplingRequest', (_message.Message,), {

  'SamplingStage' : _reflection.GeneratedProtocolMessageType('SamplingStage', (_message.Message,), {
    'DESCRIPTOR' : _RELATIONSAMPLINGREQUEST_SAMPLINGSTAGE,
    '__module__' : 'sampling_service_pb2'
    # @@protoc_insertion_point(class_scope:RelationSamplingRequest.SamplingStage)
    })
  ,

  'Relation' : _reflection.GeneratedProtocolMessageType('Relation', (_message.Message,), {
    'DESCRIPTOR' : _RELATIONSAMPLINGREQUEST_RELATION,
    '__module__' : 'sampling_service_pb2'
    # @@protoc_insertion_point(class_scope:RelationSamplingRequest.Relation)
    })
  ,

  'RelationStage' : _reflection.GeneratedProtocolMessageType('RelationStage', (_message.Message,), {
    'DESCRIPTOR' : _RELATIONSAMPLINGREQUEST_RELATIONSTAGE,
    '__module__' : 'sampling_service_pb2'
    # @@protoc_insertion_point(class_scope:RelationSamplingRequest.RelationStage)
    })
  ,
  'DESCRIPTOR' : _RELATIONSAMPLINGREQUEST,
  '__module__' : 'sampling_service_pb2'
  # @@protoc_insertion_point(class_scope:RelationSamplingRequest)
  })
_sym_db.RegisterMessage(RelationSamplingRequest)
_sym_db.RegisterMessage(RelationSamplingRequest.SamplingStage)
_sym_db.RegisterMessage(RelationSamplingRequest.Relation)
_sym_db.RegisterMessage(RelationSamplingRequest.RelationStage)

RelationSamplingResponse = _reflection.GeneratedProtocolMessageType('RelationSamplingResponse', (_message.Message,), {

  'SampleResult' : _reflection.GeneratedProtocolMessageType('SampleResult', (_message.Message,), {
    'DESCRIPTOR' : _RELATIONSAMPLINGRESPONSE_SAMPLERESULT,
    '__module__' : 'sampling_service_pb2'
    # @@protoc_insertion_point(class_scope:RelationSamplingResponse.SampleResult)
    })
  ,

  'ResponseData' : _reflection.GeneratedProtocolMessageType('ResponseData', (_message.Message,), {
    'DESCRIPTOR' : _RELATIONSAMPLINGRESPONSE_RESPONSEDATA,
    '__module__' : 'sampling_service_pb2'
    # @@protoc_insertion_point(class_scope:RelationSamplingResponse.ResponseData)
    })
  ,
  'DESCRIPTOR' : _RELATIONSAMPLINGRESPONSE,
  '__module__' : 'sampling_service_pb2'
  # @@protoc_insertion_point(class_scope:RelationSamplingResponse)
  })
_sym_db.RegisterMessage(RelationSamplingResponse)
_sym_db.RegisterMessage(RelationSamplingResponse.SampleResult)
_sym_db.RegisterMessage(RelationSamplingResponse.ResponseData)

_SPARKSAMPLINGSERVICE = DESCRIPTOR.services_by_name['SparkSamplingService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SAMPLINGMETHODTYPE._serialized_start=2112
  _SAMPLINGMETHODTYPE._serialized_end=2240
  _FILEFORMATTYPE._serialized_start=2242
  _FILEFORMATTYPE._serialized_end=2299
  _SAMPLINGREQUEST._serialized_start=27
  _SAMPLINGREQUEST._serialized_end=261
  _CANCELREQUEST._serialized_start=263
  _CANCELREQUEST._serialized_end=294
  _CANCELRESPONSE._serialized_start=296
  _CANCELRESPONSE._serialized_end=343
  _SAMPLINGRESPONSE._serialized_start=346
  _SAMPLINGRESPONSE._serialized_end=521
  _SAMPLINGRESPONSE_RESPONSEDATA._serialized_start=443
  _SAMPLINGRESPONSE_RESPONSEDATA._serialized_end=521
  _DICT._serialized_start=523
  _DICT._serialized_end=558
  _SAMPLINGCONF._serialized_start=561
  _SAMPLINGCONF._serialized_end=694
  _FILEFORMATCONF._serialized_start=696
  _FILEFORMATCONF._serialized_end=746
  _RELATIONSAMPLINGREQUEST._serialized_start=749
  _RELATIONSAMPLINGREQUEST._serialized_end=1821
  _RELATIONSAMPLINGREQUEST_SAMPLINGSTAGE._serialized_start=1132
  _RELATIONSAMPLINGREQUEST_SAMPLINGSTAGE._serialized_end=1402
  _RELATIONSAMPLINGREQUEST_RELATION._serialized_start=1405
  _RELATIONSAMPLINGREQUEST_RELATION._serialized_end=1578
  _RELATIONSAMPLINGREQUEST_RELATIONSTAGE._serialized_start=1581
  _RELATIONSAMPLINGREQUEST_RELATIONSTAGE._serialized_end=1821
  _RELATIONSAMPLINGRESPONSE._serialized_start=1824
  _RELATIONSAMPLINGRESPONSE._serialized_end=2109
  _RELATIONSAMPLINGRESPONSE_SAMPLERESULT._serialized_start=1937
  _RELATIONSAMPLINGRESPONSE_SAMPLERESULT._serialized_end=1987
  _RELATIONSAMPLINGRESPONSE_RESPONSEDATA._serialized_start=1989
  _RELATIONSAMPLINGRESPONSE_RESPONSEDATA._serialized_end=2109
  _SPARKSAMPLINGSERVICE._serialized_start=2302
  _SPARKSAMPLINGSERVICE._serialized_end=2504
# @@protoc_insertion_point(module_scope)



from sparksampling.sample.base_sampling import SparkBaseSamplingJob
import random


class SimpleSamplingImp(SparkBaseSamplingJob):
    cls_args = ['fraction', 'with_replacement', 'seed', 'count', 'sampling_col']

    def __init__(self, *args, **kwargs):
        super(SimpleSamplingImp, self).__init__(*args, **kwargs)
        self.fraction = float(kwargs.pop('fraction', 1))
        self.with_replacement = kwargs.pop('with_replacement', False)
        self.seed = kwargs.pop('seed', random.randint(1, 65535))
        self.count = kwargs.pop('count')
        self.sampling_col = list(kwargs.pop('sampling_col', []))

    def run(self, df):
        if self.sampling_col:
            df = df[self.sampling_col]
        return df.sample(
            fraction=self.fraction,
            seed=self.seed,
            withReplacement=self.with_replacement).limit(self.count)



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

requirements = [
    'grpcio>=1.35.0',
    'protobuf<4',
    'pyspark',
    'findspark',
    'traitlets',
    'pandas >= 1.2',
    'requests',
    'kubernetes',
    'boto3',
    'grpcio-tools',
    'graphlib_backport'
]

test_requirements = ['pytest>=3', 'pytest-grpc', ]

setup(
    author="wh1isper",
    author_email='9573586@qq.com',
    python_requires='>=3.5',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="pyspark-sampling",
    install_requires=requirements,
    long_description=readme,
    include_package_data=True,
    keywords='sparksampling',
    name='sparksampling',
    entry_points={
        'console_scripts': [
            'sparksampling = sparksampling.app:main',
        ],
    },
    packages=find_packages(include=['sparksampling', 'sparksampling.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/Wh1isper/pyspark-sampling',
    version='0.1.3',
    zip_safe=False,
)



from sparksampling.sample.base_sampling import SparkBaseSamplingJob
import random


class RandomSamplingImp(SparkBaseSamplingJob):
    cls_args = ['fraction', 'with_replacement', 'seed', 'count', 'sampling_col']

    def __init__(self, *args, **kwargs):
        super(RandomSamplingImp, self).__init__(*args, **kwargs)
        self.fraction = float(kwargs.pop('fraction'))
        self.with_replacement = kwargs.pop('with_replacement', False)
        self.seed = kwargs.pop('seed', random.randint(1, 65535))
        self.count = kwargs.pop('count', 0)
        self.sampling_col = list(kwargs.pop('sampling_col', []))

    def run(self, df):
        if self.sampling_col:
            df = df[self.sampling_col]
        df = df.sample(fraction=self.fraction, seed=self.seed, withReplacement=self.with_replacement)
        if self.count:
            df = df.limit(self.count)
        return df
